#include "pass_rule.h"
/**********************************
* pass_rule.c
*
* if pass_rule return TRUE, the 
* packet is not filtered.  If 
* it returns FALSE the packet is
* filtered
**********************************/
#include "../snort-1.8.1-RELEASE/snort.h"
#include "../snort-1.8.1-RELEASE/decode.h"
#include "../snort-1.8.1-RELEASE/rules.h"
#include "../snort-1.8.1-RELEASE/parser.h"
#include <pthread.h>
#include <libnet.h>
#include <dirent.h>

extern ListHead* head_tmp;

pthread_mutex_t snort_mutex = PTHREAD_MUTEX_INITIALIZER;


/********************************************************
* Create a reset packet to tear down the connection
*
* returns: number of bytes in the new packet
           pkt contains the new bytes
********************************************************/
int CreateResetPacket(PacketToProcess* ptp, Packet* p){
	int 	packet_size;
	u_char*	new_packet;
	
	packet_size=LIBNET_IP_H + LIBNET_ETH_H + LIBNET_TCP_H;
	//libnet_init_packet(packet_size, &new_packet);
	new_packet=ptp->Buffer;
	libnet_build_ethernet(p->eh->ether_src,
		p->eh->ether_dst,
		ETHERTYPE_IP,
		NULL,
		0,
		new_packet);
		
	libnet_build_ip(LIBNET_TCP_H,
		0,
		p->iph->ip_id+4,
		0,
		64,
		IPPROTO_TCP,
		*(unsigned long*)&p->iph->ip_dst,
		*(unsigned long*)&p->iph->ip_src,
		NULL,
		0,
		new_packet + LIBNET_ETH_H);
		
	libnet_build_tcp(htons(p->tcph->th_dport),
		htons(p->tcph->th_sport),
		htonl(p->tcph->th_ack),
		htonl(p->tcph->th_seq+1),
		TH_RST,
		p->tcph->th_win,
		0,
		NULL,
		0,
		new_packet + LIBNET_ETH_H + LIBNET_IP_H);
		
	/*calculate the new checksums*/
	libnet_do_checksum(new_packet+ ETH_H, IPPROTO_TCP, LIBNET_TCP_H);
	libnet_do_checksum(new_packet+ ETH_H, IPPROTO_IP, LIBNET_IP_H);
	
	//memcpy(pkt, new_packet, packet_size);
	//libnet_destroy_packet(&new_packet);
	ptp->BuffLen=packet_size;
	return TRUE;
}

/*****************************************************
* find the highest numbered rules file in the given dir
******************************************************/
int get_last_rules_filename(char* subdir, char* name, int namelen){
	DIR*			dir;
	struct dirent*	d;
	int				biggest;
	int				lastnum;
	
	dir=opendir(subdir);
	if (!dir){
		printf("Couldn't open subdir %s\n",subdir);
		return FALSE;
	}
	
	biggest=-1;
	while ((d=readdir(dir))){
		if (strncmp(d->d_name, "rules.v", 7)==0){
			lastnum=atoi(d->d_name+7);
			if (lastnum>biggest){
				sprintf(name, "%s/%s", subdir, d->d_name);
				biggest=lastnum;
			}	
		}
	}

	printf("Using filename %s\n",name);	
	closedir(dir);
	return (biggest!=-1);
}

/*****************************************************
* read the command line for anything that belonds to this
* pass rule
******************************************************/
void pass_rule_cmd_line(int argc, char* argv[]){
	int i=0;
	
	for (i=0; i<argc; i++){
		if (strncmp("-C", argv[i], 2)==0){
			if ( (i>=argc-1) || (argv[i+1][0]=="-"[0]) ){
				printf("Syntax -C <rules file dir>\n");
				exit(1);
			}
			if (!get_last_rules_filename(argv[i+1], pv.config_file, STD_BUF-1)){
				printf("Couldn't get rules filename\n");
				exit(1);
			}
			pv.use_rules=1;
			pv.rules_order_flag=1;
			OrderRuleLists("pass alert log");
			ParseRulesFile(pv.config_file, 0);
		}else if (strncmp("-c", argv[i], 2)==0){
			if ( (i>=argc-1) || (argv[i+1][0]=="-"[0]) ){
				printf("Syntax -c <rules file>\n");
				exit(1);
			}
			strncpy(pv.config_file, argv[i+1], STD_BUF-1);
			pv.use_rules=1;
			pv.rules_order_flag=1;
			OrderRuleLists("pass alert log");
			ParseRulesFile(pv.config_file, 0);
		}else if (strncasecmp("-l", argv[i], 2)==0){
			if ( (i>=argc-1) || (argv[i+1][0]=="-"[0]) ){
				printf("Syntax -l <log directory>\n");
				exit(1);
			}
			strncpy(pv.log_dir, argv[i+1], STD_BUF-1);
			pv.log_flag=1;
		}
	}
	
	if (!pv.use_rules){
		printf("You must specify a rules file with -c <rules file>\n");
		exit(1);
	}
}

/*********************************************************
* Add the rules we can't work without to snort
*********************************************************/
void add_mandatory_rules(){
	char		Rule[1024];
	ListHead*	listhead=NULL;
	
	printf("***********Adding mandatory rules****************\n");
	sprintf(Rule,"output scrub_alert: 0\n");
   	/*ParseRule(Rule,0);*/
	
	listhead = CreateRuleType("drop", RULE_ALERT, 1, NULL);
	head_tmp = listhead;
	ParseOutputPlugin(Rule);
	head_tmp=NULL;
	
	sprintf(Rule,"output scrub_alert: 1\n");
		
	listhead = CreateRuleType("sdrop", RULE_ALERT, 1, NULL);
	head_tmp = listhead;
	ParseOutputPlugin(Rule);
	head_tmp=NULL;

	sprintf(Rule,"output scrub_alert: 2\n");
		
	listhead = CreateRuleType("ignore", RULE_ALERT, 1, NULL);
	head_tmp = listhead;
	ParseOutputPlugin(Rule);
	head_tmp=NULL;
}

/***********************************************************
* initialize all the snort stuff
***********************************************************/
void pass_rule_init(pcap_t* pad, int argc, char* argv[]){
	grinders[0] = DecodeEthPkt;	
	pd=pad;	

	/*from snort main*/
	InitNetmasks();
	InitProtoNames();
	
	strncpy(pv.log_dir, "/var/log/hogwash", STD_BUF-1);
	pv.log_flag=1;
 
    InitPreprocessors();
    InitPlugIns();
    InitOutputPlugins();
	
	/* setup the default rule action anchor points */
    CreateDefaultRules();
	
	add_mandatory_rules();
	pass_rule_cmd_line(argc, argv);
		
	alert = OpenAlertFile("/var/log/hogwash/alert");
	
	pv.log_cmd_override=1;
	pv.data_flag=1;
	pv.show2hdr_flag=1;
	pv.logbin_flag=0;
}

/************************************************************
* The rules engine starts here
*************************************************************/
int pass_rule(PacketToProcess* ptp){
	Packet 	p;
	int 	return_value;

	pthread_mutex_lock(&snort_mutex);
	FilterThis=FALSE;
        Ignore=FALSE;
	(*grinder)(&p, ptp->pkthdr, ptp->pkt);
	p.interface_num=(int)ptp->user;

	Preprocess(&p);
	ClearDumpBuf();
	return_value = !FilterThis;
	pthread_mutex_unlock(&snort_mutex);
	
	
	if (!FilterThis){
		return RESPONSE_SEND_ON;
	}	
	
	if (FilterThis && p.iph && p.tcph){
		/*send back a reset to clean up the session*/
		/* If rule that triggered is not ignore */
		if (!Ignore){
			CreateResetPacket(ptp, &p);
#ifdef DEBUG
		printf("This should send back a reset\n");
#endif		
			return RESPONSE_SEND_REPLY | RESPONSE_USE_BUFFER;}
		else{
			return RESPONSE_DROP;
		}
	}else{
		/*just drop the packet*/
#ifdef DEBUG		
		printf("This packet should drop\n");
#endif		
		return RESPONSE_DROP;
	}
}

/***************************************************************
* Be nice and free up all the stuff the malloc'd
***************************************************************/
void pass_rule_free(){
}

/***************************************************************
* Restart and re-read everything
***************************************************************/
void pass_rule_restart(){
	pv.use_rules=1;
	pv.rules_order_flag=1;
	OrderRuleLists("pass alert log");
	ParseRulesFile(pv.config_file, 0);
}
